from types import MappingProxyType
from typing import Any, Dict, Type, Union, get_args, get_origin


def inject_types(type_: Any) -> None:
    if get_origin(type_) is not Union:
        return
    types_dict: Dict[str, Type[Any]] = {}
    for u_type in get_args(type_):  # TODO
        fields = getattr(u_type, "__fields__", None)
        if fields is None:
            continue
        field = fields.get("type")
        if field is None:
            continue
        type_value = getattr(field, "default", None)
        if (type_value is None) or not isinstance(type_value, str):
            continue
        types_dict[type_value] = u_type
    if types_dict:
        setattr(type_, "__types__", MappingProxyType(types_dict))



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field


# A Boolean variable in binary format.
class BooleanVariable(BaseDiv):

    def __init__(
        self, *,
        type: str = "boolean",
        name: typing.Optional[typing.Union[Expr, str]] = None,
        value: typing.Optional[typing.Union[Expr, bool]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            name=name,
            value=value,
            **kwargs,
        )

    type: str = Field(default="boolean")
    name: typing.Union[Expr, str] = Field(
        min_length=1, 
        description="Variable name.",
    )
    value: typing.Union[Expr, bool] = Field(
        description="Value.",
    )


BooleanVariable.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field


# Requests focus for element. On the web may work unstable.
class DivActionFocusElement(BaseDiv):

    def __init__(
        self, *,
        type: str = "focus_element",
        element_id: typing.Optional[typing.Union[Expr, str]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            element_id=element_id,
            **kwargs,
        )

    type: str = Field(default="focus_element")
    element_id: typing.Union[Expr, str] = Field(
        min_length=1,
    )


DivActionFocusElement.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_pivot


# Transformation of the element.
class DivTransform(BaseDiv):

    def __init__(
        self, *,
        pivot_x: typing.Optional[div_pivot.DivPivot] = None,
        pivot_y: typing.Optional[div_pivot.DivPivot] = None,
        rotation: typing.Optional[typing.Union[Expr, float]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            pivot_x=pivot_x,
            pivot_y=pivot_y,
            rotation=rotation,
            **kwargs,
        )

    pivot_x: typing.Optional[div_pivot.DivPivot] = Field(
        description="X coordinate of the rotation axis.",
    )
    pivot_y: typing.Optional[div_pivot.DivPivot] = Field(
        description="Y coordinate of the rotation axis.",
    )
    rotation: typing.Optional[typing.Union[Expr, float]] = Field(
        description=(
            "Degrees of the element rotation. Positive values used for "
            "clockwise rotation."
        ),
    )


DivTransform.update_forward_refs()
